import { Input as UIInput } from '@/components/ui/input';
import React, { useState, useRef, useEffect } from 'react';

export type InputProps = { label?: string } & React.ComponentProps<'input'>;
export const Input = (props: InputProps) => {
  return <UIInput {...props} />;
};

type TagsInputProps = {
  value: string[];
  onChange: (value: string[]) => void;
  placeholder?: string;
  label?: React.ReactNode;
  showLabel?: boolean;
  options?: string[]; // 可选，暂未实现自动补全
};

export const TagsInput = ({
  value,
  onChange,
  placeholder = '',
  label = '',
  showLabel = false,
}: TagsInputProps) => {
  const [input, setInput] = useState('');
  const inputRef = useRef<HTMLInputElement>(null);

  useEffect(() => {
    setInput('');
  }, [value]);

  const handleInputChange = (e: React.ChangeEvent<HTMLInputElement>) => {
    setInput(e.target.value);
  };

  const handleInputKeyDown = (e: React.KeyboardEvent<HTMLInputElement>) => {
    if (
      (e.key === 'Enter' || e.key === ',' || e.key === 'Tab') &&
      input.trim()
    ) {
      e.preventDefault();
      const newTag = input.trim();
      if (!value.includes(newTag)) {
        onChange([...value, newTag]);
      }
      setInput('');
    } else if (e.key === 'Backspace' && !input && value.length) {
      onChange(value.slice(0, -1));
    }
  };

  const handleRemoveTag = (idx: number) => {
    onChange(value.filter((_, i) => i !== idx));
  };

  return (
    <div>
      {showLabel && label && (
        <label className="block mb-1 text-sm font-medium text-gray-700">{label}</label>
      )}
      <div
        className="flex flex-wrap items-center gap-2 border rounded px-2 py-1 min-h-[40px] focus-within:ring-2 focus-within:ring-blue-500 bg-white"
        onClick={() => inputRef.current?.focus()}
      >
        {value.map((tag, idx) => (
          <span
            key={tag + idx}
            className="flex items-center bg-blue-100 text-blue-800 rounded px-2 py-0.5 text-sm mr-1 mb-1"
          >
            {tag}
            <button
              type="button"
              className="ml-1 text-blue-500 hover:text-blue-700 focus:outline-none"
              onClick={() => handleRemoveTag(idx)}
              aria-label="Remove tag"
            >
              ×
            </button>
          </span>
        ))}
        <input
          ref={inputRef}
          className="flex-1 min-w-[80px] border-none outline-none bg-transparent py-1 text-sm"
          value={input}
          onChange={handleInputChange}
          onKeyDown={handleInputKeyDown}
          placeholder={placeholder}
        />
      </div>
    </div>
  );
};
